﻿// --------------------------------------------------------------------------------------
//
// CDPlayerLuxus.cs                                               Ulrich Müller  31.12.07
//                                                                PCS1700 C# 2005 Express
// Luxus-Version des CD-Players aus Profi Computing 1991
//               Basis ist die Assembly FishFace2005.DLL
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using FishFace40;

namespace CDPlayerLuxus {
  public partial class frmMainLuxus : Form {
    FishFace ft = new FishFace();
    int[] Datenfeld = new int[75];

    const Out mScheibe        = Out.M1;
    const Out mLeseSchlitten  = Out.M2;
    const Out mLeseLampe      = Out.M3;
    const Inp iScheibeNull    = Inp.I1;
    const Inp iScheibeImpuls  = Inp.I2;
    const Inp iLesenImpuls    = Inp.I3;
    const Inp iLesenEnde      = Inp.I4;
    const Inp iLeseKopf       = Inp.I5;
    const Dir dVor            = Dir.Links;
    const Dir dRueck          = Dir.Rechts;

    private void Action() {
    // --- Initialisierung ---
      lblStatus.Text = "Initialisierung gestartet";
      ft.SetMotor(mLeseLampe, Dir.Ein);
      ft.SetMotor(mScheibe, Dir.Rechts);
      ft.SetMotor(mLeseSchlitten, dVor);
      ft.WaitForInput(iLesenEnde, false);
      ft.SetMotor(mLeseSchlitten, dRueck);
      ft.WaitForInput(iLesenEnde);
      ft.SetMotor(mLeseSchlitten, Dir.Aus);
      ft.WaitForInput(iScheibeNull);
      ft.SetMotor(mScheibe, Dir.Aus);
      picFisch.Visible = false;
      lstWerte.Visible = true;
      lstWerte.Items.Clear();

    // --- Scheibe Lesen ------
      ScheibeLesen(0);
      ScheibeLesen(1);
      ScheibeLesen(2);
      lblStatus.Text = "Rückstellung";
      ft.SetMotor(mLeseLampe, Dir.Aus);
      ft.SetMotor(mLeseSchlitten, dRueck);
      ft.WaitForInput(iLesenEnde);
      ft.SetMotor(mLeseSchlitten, Dir.Aus);

    // --- Tonfolge wiedergeben ----
      int Frequenz;
      int d = 0;
      string Kode;
      lstWerte.Items.Add("Dekodierte Töne");
      do {
        Frequenz = (Datenfeld[d] * 8 + Datenfeld[d + 1] * 4 +
                   Datenfeld[d + 2] * 2 + Datenfeld[d + 3]) * 100;
        if (ft.Finish()) return;
        lblStatus.Text = "Tonfrequenz : " + Frequenz.ToString();
        Kode = Datenfeld[d].ToString() + Datenfeld[d + 1].ToString() + 
               Datenfeld[d + 2].ToString() + Datenfeld[d + 3].ToString();
        lstWerte.Items.Add(Kode + "->Frequenz : " + Frequenz.ToString());
        Application.DoEvents();
        if (Frequenz > 0) Console.Beep(Frequenz, 600);
        else ft.Pause(400);
        d += 4;
      } while (d < 71);
    }

    private void ScheibeLesen(int iSpur) {
      string Spurwerte = "";
      if (iSpur > 0) {
        lblStatus.Text = "Wechsel auf Spur : " + iSpur.ToString();
        ft.SetMotor(mLeseSchlitten, dVor);
        ft.WaitForChange(iLesenImpuls, 26);
        ft.SetMotor(mLeseSchlitten, Dir.Aus);
      }
      ft.SetMotor(mScheibe, Dir.Rechts);
      for (int i = iSpur * 25; i < iSpur * 25 + 25; i++) {
        Datenfeld[i] = ft.GetInput(iLeseKopf) ? 1 : 0;
        lblStatus.Text = "Lesen bit : " + i.ToString() + "  Wert : " + 
          Datenfeld[i].ToString();
        Spurwerte += Datenfeld[i].ToString() + " ";
        ft.WaitForChange(iScheibeImpuls, 16);
      }
      ft.SetMotor(mScheibe, Dir.Aus);
      lstWerte.Items.Add("Spur : " + iSpur.ToString());
      lstWerte.Items.Add(Spurwerte);
    }

    #region --- Programm-Kontrolle ---
    public frmMainLuxus() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      // --- Open erstes ROBO an USB oder Intelligent Interface an COM
      //     während des Ablaufs von Action() gesperrt ---------------
      try {
        if (cboPortName.SelectedIndex == 0)
          ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        else ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") ft.NotHalt = true; else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
      cboPortName.SelectedIndex = 0;
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}